/* 

    izug.basetheme for seantis.reservation resources

*/

/* no longer supported in plone 4.3 */
#kss-spinner {
    display: none;   
}

/* does not work with izug.basetheme */
#ajax-spinner {
    display: none !important;   
}

.multicalendar .multiCalendarContainer {
    float: none;
    width: 100%;
}

.singlecalendar .fc, .multicalendar .fc {
    margin-top: 10px;
}

/* wizard form tabs */

.seantis-reservation-form .wizardSteps {
    height: 22px;   
    border-color: black;
    background: none;
    border: 1px solid black;
}

.seantis-reservation-form .formTabs,
.seantis-reservation-form ul.wizardSteps li.formTab {
    height: 22px;
}

.seantis-reservation-form ul.wizardSteps li.formTab a {
    margin: 0;
    padding: 1px 15px 0 5px;
    background-color: transparent;
    height: 20px;
    border: none;
}
.seantis-reservation-form ul.wizardSteps li.formTab a.selected {
    font-weight: bold;
}

.seantis-reservation-form input {
     max-width: 250px;
}

.seantis-reservation-form fieldset {
     padding: 5px 10px !important;
}

div.overlay dl.portalMessage {
     margin-top: 10px;
}

.seantis-reservation-form .formTab a.required:after {
    content: none;
}

/* inline page style */

#content #inline-page {
    border: solid 1px #bbb;
    background-color: #efefef;
    margin-top: 20px;  
    min-height: 32px;
}

#inline-page .hint {
    font-size: 12px;
    width: 100%;
    text-align: center;
    color: #555;
    padding-top:8px;
}

#content #inline-page h2 {
     margin-top: 10px;
}

/* the overlay's left offset is calculated by jquery tools and it
allows no overriding of that calculation (except for preload values),
which is a pity, because if that was possible we could center the
variable width overlay within iZug's limited #content width instead
of the window width */
div.overlay {
    width: auto;
    max-width: 880px;
    min-width: 880px;
    left: 70px !important;
}

/* Contextmenu */

#miniTip {
    background-color: white;
    border-color: #bbb;
    border-width: 1px;
    font-size: 10px;
    border-radius: 0;
}

#miniTip a {
    color: #00538c;
    font-size: 10px;
    padding: 5px 5px 5px 10px;
    display: block;
}

#miniTip a:hover, #miniTip a:visited {
    background-color: white;
    margin-left: 10px;
    padding-left: 0;
    color: #ae1e00;
    text-decoration: underline;
}

#miniTip_c {
    padding: 0;
    min-width: 100px;
}

#miniTip_c div {
    border-radius: 0;
    border: 1px solid #dedede;
    border-width: 1px 0 1px 0;
    background-color: #efefef;
    color: black;
    line-height: 20px;
    padding: 0 10px 0 10px;
}

#miniTip_c div:first-child {
    border-top: none;   
}

#miniTip .n {
    border-bottom-color: #00538c;
}

#miniTip .s {
    border-bottom-color: #00538c;
}

#miniTip .e {
    border-right-color: #00538c;
}

#miniTip .w {
    border-left-color: #00538c;
}

/* Your reservations */
.your-reservations {
    padding: 0 10px 10px 10px;
    border: 1px solid #ddd;
    margin-top: 10px;
    margin-bottom: 20px;
    background-color: #ebf4fb;
}

.your-reservations h2 {
    border-bottom: 2px solid #006fba;
    padding-bottom: 10px;
    margin-top: 10px;
}

.your-reservations-view ul li,
.your-reservations ul li {
    border-bottom: 1px solid #99C5E3;
    padding-bottom: 8px;
}

.your-reservations-view ul {
    margin-top: -10px;
    background-color: #ebf4fb;   
}
.your-reservations-view ul li {
    margin: 0;
    padding: 2px 0 4px 15px;
}

.your-reservations-view ul,
.your-reservations ul {
    list-style: none;
    margin-left: 0;
}

.your-reservations-view .formControls,
.your-reservations .formControls {
    margin-left: -2px;
}

.your-reservations ul span {
    display: block;
}

.your-reservations ul a {
    color: black !important;
}

.your-reservations ul a.remove-url {
    float: right;
    color: #ad1d0a !important;
}

.your-reservations ul a:hover {
    text-decoration: underline !important;
}

.facilityOverview .your-reservations {
    margin-top: -2px;
    margin-bottom: 10px;
}

.portaltype-seantis-dir-facility-item .facilityOverview .your-reservations {
    margin-top: 5px;
}

/* reservations in progress */
.uncommitted {
    color: #555;
    font-size: 12px;
    margin-top: -26px;
    margin-bottom: 11px;
}

/* form fixes */
.seantis-reservation-form #formfield-form-widgets-id,
.seantis-reservation-form #formfield-form-widgets-group,
.seantis-reservation-form #formfield-form-widgets-timeframes {
   display: none;   
}

#form-widgets-description {
   width: 932px;   
}
#form-widgets-first_hour,
#form-widgets-last_hour {
   width: 936px;   
}

#formfield-form-widgets-days span.option {
    display: inline;
}

/* izug basetheme applies div.error to nested divs for some reason, this
change fixes that */
div.field.error {
    background-color: white;
    border: none;
    padding-left: 0;
    margin-top: 5px;
}

/* form help */
.field {
    padding-top: 5px;
}
.wizard .field {
    padding: 0;
}
.formHelp {
   font-weight: normal;
   display: block;
   margin: 0 0 4px 0 !important;
   line-height: 12px;
   font-size: 10px;
   
   color: #454545;
}
.seantis-reservation-form .formHelp {
    width: 253px;
    text-align: justify;
}

/* reservations management */
.reservation {
    margin-bottom: 20px;
}

.reservation-list {
    padding-bottom: 20px;
}

.reservation-title {
    width: 100%; 
    height: 25px;
    margin-top: 20px;
    padding-top: 5px;
}

.reservation-pending .reservation-title {
    background-color: #00538C;
}

.reservation-approved .reservation-title {
    background-color: #454545;
}

.reservation-title h3,
#content .reservation-title a,
#content .reservation-title a:visited,
#content .reservation-info .reservation-title a,
#content .reservation-info .reservation-title a:visited {
    font-size: 1em !important;
    font-weight: normal !important;
    display: inline;
    margin-left: 10px;
    margin-right: 11px;
    color: white !important;
    border-color: white !important;
}

#content .reservation-info .reservation-title a:hover,
#content .reservation-title a:hover {
    border-bottom-style: solid !important;
}

#content .reservation-info {
    margin: -10px auto 10px auto;   
}

#content .inline-reservations {
    border-top: solid 1px #dedede;
    margin: 0;
    padding: 0 10px;
    background-color: white;     
}

a.reservation-delete {
   float: right;
}

a.reservation-data-update {
    display: inline-block;
    margin-top: 10px;
}

.reservation-list {
    border: 1px solid #ccc;
    border-top: none;
    padding: 5px 10px 5px 20px;
}

#content .inline-reservations {
    border-color: #bbb;
    min-height: 50px;
}

.reservation-approved .reservation-title,
.reservation-pending .reservation-title {
    font-size: 12px;
    height: 23px;
}

#ftw-edit-bar {
    position: relative !important;
    z-index: 1000 !important;
}

label .required {
    background-image: url("/++resource++seantis.reservation.images/required.gif");
    background-position: right center;
    background-repeat: no-repeat;
    padding-right: 3px;
}

/* reservation data */
.reservations-by-month .reservation-data {
     float:none;   
     width: 100%;
}

.reservation-data, .reservation-timespans {
     float:left;
     width: 50%;
}

.reservation-data {
    line-height: 1.35em;
    font-size: 11px;
    padding-bottom: 5px;
}

.reservation-data span.label {
    font-weight: normal;
    color: #454545;
    display: inline-block;
    min-width: 75px;
    margin-top: 3px;
}

.reservation-list .block-title {
    font-size: 1em;
    font-weight: bold;   
    color: #222;
    padding-top: 15px;
    padding-bottom: 1px;
}

/* reservations quota */
.reservation-quota {
    border: 1px solid #ccc;
    border-width: 0 1px 1px 1px;
    margin-top: -20px;
    height: 20px;
    background-color: #efefef;
    line-height: 20px;
    padding: 0 0 0 20px;
}

.your-reservation-quota {
    display: block !important;
}

.your-reservation-quota b {
    font-weight: normal;
}

/* group reservation list */
.groupListDay {
    float: left;
    width: 75px;
    margin-bottom: 10px;
}

.groupListTime {
    width:300px !important; 
    float:left;
    text-align: center;
}

/* reservation exports */
.reservation-exports ul li {
    list-style: none;
}

.reservation-exports ul ul {
    margin-left: 0;   
    margin-top: 0.5em;
}

.seantis-reservation-form input[readonly="readonly"] {
    background-color: #efefef;
}